/********************************************************/
/*                                                      */
/*               OO7 Benchmark                          */
/*                                                      */
/*              COPYRIGHT (C) 1993                      */
/*                                                      */
/*                Michael J. Carey 		        */
/*                David J. DeWitt 		        */
/*                Jeffrey Naughton 		        */
/*               Madison, WI U.S.A.                     */
/*                                                      */
/*	         ALL RIGHTS RESERVED                    */
/*                                                      */
/********************************************************/


////////////////////////////////////////////////////////////////////////////
//
// Assembly Methods
//
////////////////////////////////////////////////////////////////////////////

#include <libc.h>
#include <string.h>

#include "OO7.hxx"
#include "GenParams.h"
#include "VarParams.h"
#include "baidlist.h"

extern int	nextComplexAssemblyId;
extern int	nextBaseAssemblyId;
extern int	debugMode;
extern char*	types[NumTypes];

extern BAIdList* private_cp;
extern BAIdList* shared_cp;

int    numCompositePartsLinked = 0;


////////////////////////////////////////////////////////////////////////////
//
// Assembly Constructor
//
////////////////////////////////////////////////////////////////////////////

Assembly::Assembly(int asId) : id(asId) { }


//////////////////////////////////////////////////////////////////////////
// 
// DoNothing method for Assemblies 
// 
//////////////////////////////////////////////////////////////////////////

void Assembly::DoNothing ()
{
  if (id < 0) 
    printf("DoNothing - negative id\n");

  if (debugMode) {
    printf("==> DoNothing(id = %d, type = %s, buildDate = %d)\n",
	   id, type, buildDate);
  }
}

////////////////////////////////////////////////////////////////////////////
//
// ComplexAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

ComplexAssembly::ComplexAssembly(int asId,
    	                         ComplexAssembly* parentAssembly,
		                 int levelNo,
				 Module *mod
				 ) : Assembly(asId)
{
  if (debugMode) {
    printf("ComplexAssembly::ComplexAssembly(asId = %d, levelNo = %d)\n",
	   asId, levelNo);
  }

  // initialize internal state of new assembly (some of it randomly)

  int typeNo = (int) (random() % NumTypes);
  strncpy(type, types[typeNo], TypeSize);
  buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));
  superAssembly = parentAssembly;
  if (parentAssembly) parentAssembly->subAssemblies += this ;
  module = mod;
  module->assemblies += this ;

  // now recursively create subassemblies for this complex assembly

  int i, nextId;

  for (i = 0; i < NumAssmPerAssm; i++) {

    if (levelNo < NumAssmLevels-1) {
      nextId = nextComplexAssemblyId++;
      new (this) ComplexAssembly (nextId, this, levelNo+1, mod);
    } else {
      nextId = nextBaseAssemblyId++;
      new (this) BaseAssembly (nextId, this, mod);
    }
  }
}


////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Constructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::BaseAssembly(int asId,
    	                   ComplexAssembly* parentAssembly,
			   Module *mod
			  ) : Assembly(asId)
{
    if (debugMode) {
        printf("BaseAssembly::BaseAssembly(asId = %d, levelNo = %d)\n",
                asId, NumAssmLevels);
    }

    // initialize internal state of new assembly (some of it randomly)
    int typeNo = (int) (random() % NumTypes);
    strncpy(type, types[typeNo], TypeSize);
    buildDate = MinAssmDate + (int) (random() % (MaxAssmDate-MinAssmDate+1));
    superAssembly = parentAssembly;
    if (parentAssembly) parentAssembly->subAssemblies += this ;
    module = mod;
    module->assemblies += this ;

    // add part to BaseAssembly extent
    BaseAssembly::extent->insert(this);

    // select the private composite parts for this assembly

    int i;
    int compId;
    int lowCompId = (module->id - 1) * NumCompPerModule + 1;
    int compIdLimit = NumCompPerModule;
    CompositePart* comp;

    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = lowCompId + (int) (random() % compIdLimit);
	private_cp[compId].insert(this) ;  // keep track of which
		// composite parts this base assembly uses as private
    }

    // now select the shared composite parts for this assembly
    for (i = 0; i < NumCompPerAssm; i++) 
    {
	compId = (int) (random() % TotalCompParts) + 1;
	shared_cp[compId].insert(this) ;  // keep track of which
		// composite parts this base assembly uses as shared
    }
}


////////////////////////////////////////////////////////////////////////////
//
// BaseAssembly Destructor
//
////////////////////////////////////////////////////////////////////////////

BaseAssembly::~BaseAssembly()
{
    if (debugMode) {
        printf("BaseAssembly::~BaseAssembly(id = %d)\n", id);
    }
 
    // remove part from BaseAssembly extent
    BaseAssembly::extent->remove(this) ;
}

